@@ -1,4 +1,4 @@
-pub use crate::p1::{Error, Result};
+pub use ftd::p1::{Error, Result};
 
 #[derive(Debug, PartialEq, Default, Clone, serde::Serialize, serde::Deserialize)]
 pub struct Header(pub Vec<(usize, String, String)>);
@@ -53,7 +53,7 @@ impl Header {
                 return if v == "true" || v == "false" {
                     Ok(v == "true")
                 } else {
-                    Err(crate::p1::Error::ParseError {
+                    Err(ftd::p1::Error::ParseError {
                         message: "can't parse bool".to_string(),
                         doc_id: doc_id.to_string(),
                         line_number: *l,
@@ -206,8 +206,8 @@ impl Header {
     pub fn get_events(
         &self,
         line_number: usize,
-        doc: &crate::p2::TDoc,
-        arguments: &std::collections::BTreeMap<String, crate::p2::Kind>,
+        doc: &ftd::p2::TDoc,
+        arguments: &std::collections::BTreeMap<String, ftd::p2::Kind>,
     ) -> ftd::p1::Result<Vec<ftd::p2::Event>> {
         let events = {
             let mut events = vec![];
@@ -251,18 +251,18 @@ impl Header {
         doc_id: &str,
         line_number: usize,
         name: &str,
-    ) -> Result<Vec<(String, Option<&str>)>> {
+    ) -> Result<Vec<(usize, String, Option<&str>)>> {
         let mut conditional_vector = vec![];
-        for (_, k, v) in self.0.iter() {
+        for (idx, (_, k, v)) in self.0.iter().enumerate() {
             if k == name {
-                conditional_vector.push((v.to_string(), None));
+                conditional_vector.push((idx, v.to_string(), None));
             }
             if k.contains(" if ") {
                 let mut parts = k.splitn(2, " if ");
                 let property_name = parts.next().unwrap().trim();
                 if property_name == name {
                     let conditional_attribute = parts.next().unwrap().trim();
-                    conditional_vector.push((v.to_string(), Some(conditional_attribute)));
+                    conditional_vector.push((idx, v.to_string(), Some(conditional_attribute)));
                 }
             }
         }
