@@ -17,111 +17,154 @@ impl<'a> TDoc<'a> {
         })?;
 
         if let Ok(v) = self.get_value(0, section.name.as_str()) {
-            return from_json_(section.line_number, self, &json, v.kind());
+            return self.from_json_(section.line_number, &json, v.kind());
         }
         if let Ok(list) = ftd::Variable::list_from_p1(section, self) {
-            return from_json_(section.line_number, self, &json, list.value.kind());
+            return self.from_json_(section.line_number, &json, list.value.kind());
         }
         if let Ok(var) = ftd::Variable::from_p1(section, self) {
-            return from_json_(section.line_number, self, &json, var.value.kind());
+            return self.from_json_(section.line_number, &json, var.value.kind());
         }
 
-        return ftd::e2(
+        ftd::e2(
             "component should be var or list",
             self.name,
             section.line_number,
-        );
+        )
+    }
 
-        fn from_json_(
+    fn from_json_(
+        &self,
+        line_number: usize,
+        json: &serde_json::Value,
+        kind: ftd::p2::Kind,
+    ) -> ftd::p1::Result<ftd::Value> {
+        Ok(match kind {
+            ftd::p2::Kind::String { .. } => ftd::Value::String {
+                text: serde_json::from_value::<String>(json.to_owned()).map_err(|_| {
+                    ftd::p1::Error::ParseError {
+                        message: format!("Can't parse to string, found: {}", json),
+                        doc_id: self.name.to_string(),
+                        line_number,
+                    }
+                })?,
+                source: ftd::TextSource::Header,
+            },
+            ftd::p2::Kind::Integer { .. } => ftd::Value::Integer {
+                value: serde_json::from_value::<i64>(json.to_owned()).map_err(|_| {
+                    ftd::p1::Error::ParseError {
+                        message: format!("Can't parse to integer, found: {}", json),
+                        doc_id: self.name.to_string(),
+                        line_number,
+                    }
+                })?,
+            },
+            ftd::p2::Kind::Decimal { .. } => ftd::Value::Decimal {
+                value: serde_json::from_value::<f64>(json.to_owned()).map_err(|_| {
+                    ftd::p1::Error::ParseError {
+                        message: format!("Can't parse to decimal, found: {}", json),
+                        doc_id: self.name.to_string(),
+                        line_number,
+                    }
+                })?,
+            },
+            ftd::p2::Kind::Boolean { .. } => ftd::Value::Boolean {
+                value: serde_json::from_value::<bool>(json.to_owned()).map_err(|_| {
+                    ftd::p1::Error::ParseError {
+                        message: format!("Can't parse to boolean,found: {}", json),
+                        doc_id: self.name.to_string(),
+                        line_number,
+                    }
+                })?,
+            },
+            ftd::p2::Kind::Record { name, .. } => {
+                let rec_fields = self.get_record(line_number, &name)?.fields;
+                let mut fields: std::collections::BTreeMap<String, ftd::PropertyValue> =
+                    Default::default();
+                if let serde_json::Value::Object(o) = json {
+                    for (key, kind) in rec_fields {
+                        let val = match o.get(&key) {
+                            Some(v) => v,
+                            None => {
+                                return ftd::e2(
+                                    format!("key not found: {}", key.as_str()),
+                                    self.name,
+                                    line_number,
+                                )
+                            }
+                        };
+                        fields.insert(
+                            key,
+                            ftd::PropertyValue::Value {
+                                value: self.from_json_(line_number, val, kind)?,
+                            },
+                        );
+                    }
+                } else {
+                    return ftd::e2(
+                        format!("expected object of record type, found: {}", json),
+                        self.name,
+                        line_number,
+                    );
+                }
+                ftd::Value::Record { name, fields }
+            }
+            ftd::p2::Kind::List { kind } => {
+                let kind = kind.as_ref();
+                let mut data: Vec<ftd::Value> = vec![];
+                if let serde_json::Value::Array(list) = json {
+                    for item in list {
+                        data.push(self.from_json_(line_number, item, kind.to_owned())?);
+                    }
+                } else {
+                    return ftd::e2(
+                        format!("expected object of list type, found: {}", json),
+                        self.name,
+                        line_number,
+                    );
+                }
+                ftd::Value::List {
+                    data,
+                    kind: kind.to_owned(),
+                }
+            }
+            t => unimplemented!(
+                "{:?} not yet implemented, line number: {}, doc: {}",
+                t,
+                line_number,
+                self.name.to_string()
+            ),
+        })
+    }
+
+    pub fn from_json_rows(
+        &self,
+        section: &ftd::p1::Section,
+        rows: &[Vec<serde_json::Value>],
+    ) -> ftd::p1::Result<ftd::Value> {
+        if let Ok(v) = self.get_value(0, section.name.as_str()) {
+            return from_json_rows_(section.line_number, self, rows, v.kind());
+        }
+        if let Ok(list) = ftd::Variable::list_from_p1(section, self) {
+            return from_json_rows_(section.line_number, self, rows, list.value.kind());
+        }
+
+        return ftd::e2("component should be list", self.name, section.line_number);
+
+        fn from_json_rows_(
             line_number: usize,
-            doc: &TDoc,
-            json: &serde_json::Value,
+            doc: &ftd::p2::TDoc,
+            rows: &[Vec<serde_json::Value>],
             kind: ftd::p2::Kind,
         ) -> ftd::p1::Result<ftd::Value> {
             Ok(match kind {
-                ftd::p2::Kind::String { .. } => ftd::Value::String {
-                    text: serde_json::from_value::<String>(json.to_owned()).map_err(|_| {
-                        ftd::p1::Error::ParseError {
-                            message: format!("Can't parse to string, found: {}", json),
-                            doc_id: doc.name.to_string(),
-                            line_number,
-                        }
-                    })?,
-                    source: ftd::TextSource::Header,
-                },
-                ftd::p2::Kind::Integer { .. } => ftd::Value::Integer {
-                    value: serde_json::from_value::<i64>(json.to_owned()).map_err(|_| {
-                        ftd::p1::Error::ParseError {
-                            message: format!("Can't parse to integer, found: {}", json),
-                            doc_id: doc.name.to_string(),
-                            line_number,
-                        }
-                    })?,
-                },
-                ftd::p2::Kind::Decimal { .. } => ftd::Value::Decimal {
-                    value: serde_json::from_value::<f64>(json.to_owned()).map_err(|_| {
-                        ftd::p1::Error::ParseError {
-                            message: format!("Can't parse to decimal, found: {}", json),
-                            doc_id: doc.name.to_string(),
-                            line_number,
-                        }
-                    })?,
-                },
-                ftd::p2::Kind::Boolean { .. } => ftd::Value::Boolean {
-                    value: serde_json::from_value::<bool>(json.to_owned()).map_err(|_| {
-                        ftd::p1::Error::ParseError {
-                            message: format!("Can't parse to boolean,found: {}", json),
-                            doc_id: doc.name.to_string(),
-                            line_number,
-                        }
-                    })?,
-                },
-                ftd::p2::Kind::Record { name, .. } => {
-                    let rec_fields = doc.get_record(line_number, &name)?.fields;
-                    let mut fields: std::collections::BTreeMap<String, ftd::PropertyValue> =
-                        Default::default();
-                    if let serde_json::Value::Object(o) = json {
-                        for (key, kind) in rec_fields {
-                            let val = match o.get(&key) {
-                                Some(v) => v,
-                                None => {
-                                    return ftd::e2(
-                                        format!("key not found: {}", key.as_str()),
-                                        doc.name,
-                                        line_number,
-                                    )
-                                }
-                            };
-                            fields.insert(
-                                key,
-                                ftd::PropertyValue::Value {
-                                    value: from_json_(line_number, doc, val, kind)?,
-                                },
-                            );
-                        }
-                    } else {
-                        return ftd::e2(
-                            format!("expected object of record type, found: {}", json),
-                            doc.name,
-                            line_number,
-                        );
-                    }
-                    ftd::Value::Record { name, fields }
-                }
                 ftd::p2::Kind::List { kind } => {
                     let kind = kind.as_ref();
                     let mut data: Vec<ftd::Value> = vec![];
-                    if let serde_json::Value::Array(list) = json {
-                        for item in list {
-                            data.push(from_json_(line_number, doc, item, kind.to_owned())?);
-                        }
-                    } else {
-                        return ftd::e2(
-                            format!("expected object of list type, found: {}", json),
-                            doc.name,
-                            line_number,
-                        );
+                    for row in rows {
+                        data.push(doc.from_json_row_(line_number, row, kind.to_owned())?);
                     }
+
                     ftd::Value::List {
                         data,
                         kind: kind.to_owned(),
@@ -137,6 +180,74 @@ impl<'a> TDoc<'a> {
         }
     }
 
+    pub fn from_json_row(
+        &self,
+        section: &ftd::p1::Section,
+        row: &[serde_json::Value],
+    ) -> ftd::p1::Result<ftd::Value> {
+        if let Ok(v) = self.get_value(0, section.name.as_str()) {
+            return self.from_json_row_(section.line_number, row, v.kind());
+        }
+        if let Ok(var) = ftd::Variable::from_p1(section, self) {
+            return self.from_json_row_(section.line_number, row, var.value.kind());
+        }
+
+        ftd::e2(
+            "component should be var of record type",
+            self.name,
+            section.line_number,
+        )
+    }
+
+    fn from_json_row_(
+        &self,
+        line_number: usize,
+        row: &[serde_json::Value],
+        kind: ftd::p2::Kind,
+    ) -> ftd::p1::Result<ftd::Value> {
+        Ok(match kind {
+            ftd::p2::Kind::Record { name, .. } => {
+                let rec = self.get_record(line_number, &name)?;
+                let rec_fields = rec.fields;
+                let mut fields: std::collections::BTreeMap<String, ftd::PropertyValue> =
+                    Default::default();
+                for (idx, key) in rec.order.iter().enumerate() {
+                    if let Some(kind) = rec_fields.get(key) {
+                        let val = match row.get(idx) {
+                            Some(v) => v,
+                            None => {
+                                return ftd::e2(
+                                    format!("key not found: {}", key.as_str()),
+                                    self.name,
+                                    line_number,
+                                )
+                            }
+                        };
+                        fields.insert(
+                            key.to_string(),
+                            ftd::PropertyValue::Value {
+                                value: self.from_json_(line_number, val, kind.to_owned())?,
+                            },
+                        );
+                    } else {
+                        return ftd::e2(
+                            format!("field `{}` not found", key),
+                            self.name,
+                            line_number,
+                        );
+                    }
+                }
+                ftd::Value::Record { name, fields }
+            }
+            t => unimplemented!(
+                "{:?} not yet implemented, line number: {}, doc: {}",
+                t,
+                line_number,
+                self.name.to_string()
+            ),
+        })
+    }
+
     pub fn format_name(&self, name: &str) -> String {
         format!("{}#{}", self.name, name)
     }
@@ -483,3 +594,139 @@ impl<'a> TDoc<'a> {
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    #[test]
+    fn from_rows() {
+        let source = indoc::indoc!(
+            "
+            -- record person:
+            string name:
+            integer age:
+            string address:
+            string bio:
+            "
+        )
+        .to_string();
+
+        let (g_bag, _g_col) =
+            ftd::p2::interpreter::interpret("foo/bar", &source, &ftd::p2::TestLibrary {})
+                .expect("found error");
+        let data: Vec<Vec<serde_json::Value>> = vec![
+            vec![
+                serde_json::json!("Amitu"),
+                serde_json::json!(20),
+                serde_json::json!("Bangalore"),
+                serde_json::json!("CEO of fifthTry"),
+            ],
+            vec![
+                serde_json::json!("Arpita"),
+                serde_json::json!(20),
+                serde_json::json!("Varanasi"),
+                serde_json::json!("Software Developer of fifthTry"),
+            ],
+        ];
+        let doc = ftd::p2::TDoc {
+            name: "foo/bar",
+            aliases: &Default::default(),
+            bag: &g_bag,
+        };
+        let section = ftd::p1::parse(
+            indoc::indoc!(
+                "
+            -- person list people:
+            "
+            ),
+            "foo/bar",
+        )
+        .unwrap();
+        let value_from_json = doc.from_json_rows(&section[0], &data).unwrap();
+        let value = ftd::Value::List {
+            data: vec![
+                ftd::Value::Record {
+                    name: "foo/bar#person".to_string(),
+                    fields: std::array::IntoIter::new([
+                        (
+                            "name".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::String {
+                                    text: "Amitu".to_string(),
+                                    source: ftd::TextSource::Header,
+                                },
+                            },
+                        ),
+                        (
+                            "age".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::Integer { value: 20 },
+                            },
+                        ),
+                        (
+                            "bio".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::String {
+                                    text: "CEO of fifthTry".to_string(),
+                                    source: ftd::TextSource::Header,
+                                },
+                            },
+                        ),
+                        (
+                            "address".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::String {
+                                    text: "Bangalore".to_string(),
+                                    source: ftd::TextSource::Header,
+                                },
+                            },
+                        ),
+                    ])
+                    .collect(),
+                },
+                ftd::Value::Record {
+                    name: "foo/bar#person".to_string(),
+                    fields: std::array::IntoIter::new([
+                        (
+                            "name".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::String {
+                                    text: "Arpita".to_string(),
+                                    source: ftd::TextSource::Header,
+                                },
+                            },
+                        ),
+                        (
+                            "age".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::Integer { value: 20 },
+                            },
+                        ),
+                        (
+                            "bio".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::String {
+                                    text: "Software Developer of fifthTry".to_string(),
+                                    source: ftd::TextSource::Header,
+                                },
+                            },
+                        ),
+                        (
+                            "address".to_string(),
+                            ftd::PropertyValue::Value {
+                                value: ftd::Value::String {
+                                    text: "Varanasi".to_string(),
+                                    source: ftd::TextSource::Header,
+                                },
+                            },
+                        ),
+                    ])
+                    .collect(),
+                },
+            ],
+            kind: ftd::p2::Kind::Record {
+                name: "foo/bar#person".to_string(),
+            },
+        };
+        pretty_assertions::assert_eq!(value_from_json, value);
+    }
+}
