@@ -2,7 +2,7 @@ pub fn parse_import(
     c: &Option<String>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<(String, String)> {
+) -> ftd::p1::Result<(String, String)> {
     let v = match c {
         Some(v) => v.trim(),
         None => {
@@ -40,12 +40,12 @@ pub fn parse_import(
 pub fn string_and_ref(
     line_number: usize,
     name: &str,
-    properties: &std::collections::BTreeMap<String, (crate::Value, Option<String>)>,
+    properties: &std::collections::BTreeMap<String, (ftd::Value, Option<String>)>,
     all_locals: &mut ftd::Map,
     doc_id: &str,
-) -> crate::p1::Result<(String, Option<String>)> {
+) -> ftd::p1::Result<(String, Option<String>)> {
     match properties.get(name) {
-        Some((crate::Value::String { text, .. }, reference)) => {
+        Some((ftd::Value::String { text, .. }, reference)) => {
             Ok((text.to_string(), complete_reference(reference, all_locals)))
         }
         Some(v) => ftd::e2(
@@ -60,12 +60,12 @@ pub fn string_and_ref(
 pub fn string_and_source_and_ref(
     line_number: usize,
     name: &str,
-    properties: &std::collections::BTreeMap<String, (crate::Value, Option<String>)>,
+    properties: &std::collections::BTreeMap<String, (ftd::Value, Option<String>)>,
     all_locals: &mut ftd::Map,
     doc_id: &str,
-) -> crate::p1::Result<(String, crate::TextSource, Option<String>)> {
+) -> ftd::p1::Result<(String, ftd::TextSource, Option<String>)> {
     match properties.get(name) {
-        Some((crate::Value::String { text, source }, reference)) => Ok((
+        Some((ftd::Value::String { text, source }, reference)) => Ok((
             text.to_string(),
             source.to_owned(),
             complete_reference(reference, all_locals),
@@ -100,14 +100,14 @@ pub fn complete_reference(reference: &Option<String>, all_locals: &mut ftd::Map)
 
 pub fn string_optional(
     name: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<Option<String>> {
+) -> ftd::p1::Result<Option<String>> {
     match properties.get(name) {
-        Some(crate::Value::String { text: v, .. }) => Ok(Some(v.to_string())),
-        Some(crate::Value::None {
-            kind: crate::p2::Kind::String { .. },
+        Some(ftd::Value::String { text: v, .. }) => Ok(Some(v.to_string())),
+        Some(ftd::Value::None {
+            kind: ftd::p2::Kind::String { .. },
         }) => Ok(None),
         Some(ftd::Value::None { .. }) => Ok(None),
         Some(v) => ftd::e2(
@@ -122,14 +122,14 @@ pub fn string_optional(
 pub fn string_with_default(
     name: &str,
     def: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<String> {
+) -> ftd::p1::Result<String> {
     match properties.get(name) {
-        Some(crate::Value::String { text: v, .. }) => Ok(v.to_string()),
-        Some(crate::Value::None {
-            kind: crate::p2::Kind::String { .. },
+        Some(ftd::Value::String { text: v, .. }) => Ok(v.to_string()),
+        Some(ftd::Value::None {
+            kind: ftd::p2::Kind::String { .. },
         }) => Ok(def.to_string()),
         Some(ftd::Value::None { .. }) => Ok(def.to_string()),
         Some(v) => ftd::e2(
@@ -143,12 +143,12 @@ pub fn string_with_default(
 
 pub fn int(
     name: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<i64> {
+) -> ftd::p1::Result<i64> {
     match properties.get(name) {
-        Some(crate::Value::Integer { value: v, .. }) => Ok(*v),
+        Some(ftd::Value::Integer { value: v, .. }) => Ok(*v),
         Some(v) => ftd::e2(
             format!("[{}] expected int, found: {:?}", name, v),
             doc_id,
@@ -160,14 +160,14 @@ pub fn int(
 
 pub fn int_optional(
     name: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<Option<i64>> {
+) -> ftd::p1::Result<Option<i64>> {
     match properties.get(name) {
-        Some(crate::Value::Integer { value: v }) => Ok(Some(*v)),
-        Some(crate::Value::None {
-            kind: crate::p2::Kind::Integer { .. },
+        Some(ftd::Value::Integer { value: v }) => Ok(Some(*v)),
+        Some(ftd::Value::None {
+            kind: ftd::p2::Kind::Integer { .. },
         }) => Ok(None),
         Some(ftd::Value::None { .. }) => Ok(None),
         Some(v) => ftd::e2(format!("expected int, found: {:?}", v), doc_id, line_number),
@@ -178,14 +178,14 @@ pub fn int_optional(
 pub fn int_with_default(
     name: &str,
     def: i64,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<i64> {
+) -> ftd::p1::Result<i64> {
     match properties.get(name) {
-        Some(crate::Value::Integer { value: v }) => Ok(*v),
-        Some(crate::Value::None {
-            kind: crate::p2::Kind::Integer { .. },
+        Some(ftd::Value::Integer { value: v }) => Ok(*v),
+        Some(ftd::Value::None {
+            kind: ftd::p2::Kind::Integer { .. },
         }) => Ok(def),
         Some(ftd::Value::None { .. }) => Ok(def),
         Some(v) => ftd::e2(format!("expected int, found: {:?}", v), doc_id, line_number),
@@ -195,26 +195,26 @@ pub fn int_with_default(
 
 // pub fn elements(
 //     name: &str,
-//     properties: &std::collections::BTreeMap<String, crate::Value>,
-// ) -> crate::p1::Result<Vec<ftd::Element>> {
+//     properties: &std::collections::BTreeMap<String, ftd::Value>,
+// ) -> ftd::p1::Result<Vec<ftd::Element>> {
 //     match properties.get(name) {
-//         Some(crate::Value::Elements(v)) => Ok((*v).clone()),
-//         Some(v) => crate::e(format!("expected elements, found: {:?}", v)),
-//         None => crate::e(format!("'{}' not found", name)),
+//         Some(ftd::Value::Elements(v)) => Ok((*v).clone()),
+//         Some(v) => ftd::e(format!("expected elements, found: {:?}", v)),
+//         None => ftd::e(format!("'{}' not found", name)),
 //     }
 // }
 
 pub fn bool_with_default(
     name: &str,
     def: bool,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<bool> {
+) -> ftd::p1::Result<bool> {
     match properties.get(name) {
-        Some(crate::Value::Boolean { value: v }) => Ok(*v),
-        Some(crate::Value::None {
-            kind: crate::p2::Kind::Boolean { .. },
+        Some(ftd::Value::Boolean { value: v }) => Ok(*v),
+        Some(ftd::Value::None {
+            kind: ftd::p2::Kind::Boolean { .. },
         }) => Ok(def),
         Some(ftd::Value::None { .. }) => Ok(def),
         Some(v) => ftd::e2(
@@ -228,12 +228,12 @@ pub fn bool_with_default(
 
 pub fn bool(
     name: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<bool> {
+) -> ftd::p1::Result<bool> {
     match properties.get(name) {
-        Some(crate::Value::Boolean { value: v, .. }) => Ok(*v),
+        Some(ftd::Value::Boolean { value: v, .. }) => Ok(*v),
         Some(v) => ftd::e2(
             format!("[{}] expected bool, found: {:?}", name, v),
             doc_id,
@@ -245,17 +245,17 @@ pub fn bool(
 
 pub fn string_bool_optional(
     name: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<(Option<bool>, Option<String>)> {
+) -> ftd::p1::Result<(Option<bool>, Option<String>)> {
     match properties.get(name) {
-        Some(crate::Value::Boolean { value: v }) => Ok((Some(*v), None)),
-        Some(crate::Value::None {
-            kind: crate::p2::Kind::Boolean { .. },
+        Some(ftd::Value::Boolean { value: v }) => Ok((Some(*v), None)),
+        Some(ftd::Value::None {
+            kind: ftd::p2::Kind::Boolean { .. },
         }) => Ok((None, None)),
         Some(ftd::Value::None { .. }) => Ok((None, None)),
-        Some(crate::Value::String { text: v, .. }) => {
+        Some(ftd::Value::String { text: v, .. }) => {
             if let Ok(b) = v.parse::<bool>() {
                 Ok((Some(b), None))
             } else {
@@ -296,12 +296,12 @@ mod test {
 
 pub fn decimal(
     name: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<f64> {
+) -> ftd::p1::Result<f64> {
     match properties.get(name) {
-        Some(crate::Value::Decimal { value: v, .. }) => Ok(*v),
+        Some(ftd::Value::Decimal { value: v, .. }) => Ok(*v),
         Some(v) => ftd::e2(
             format!("[{}] expected Decimal, found: {:?}", name, v),
             doc_id,
@@ -313,14 +313,14 @@ pub fn decimal(
 
 pub fn decimal_optional(
     name: &str,
-    properties: &std::collections::BTreeMap<String, crate::Value>,
+    properties: &std::collections::BTreeMap<String, ftd::Value>,
     doc_id: &str,
     line_number: usize,
-) -> crate::p1::Result<Option<f64>> {
+) -> ftd::p1::Result<Option<f64>> {
     match properties.get(name) {
-        Some(crate::Value::Decimal { value: v }) => Ok(Some(*v)),
-        Some(crate::Value::None {
-            kind: crate::p2::Kind::Decimal { .. },
+        Some(ftd::Value::Decimal { value: v }) => Ok(Some(*v)),
+        Some(ftd::Value::None {
+            kind: ftd::p2::Kind::Decimal { .. },
         }) => Ok(None),
         Some(ftd::Value::None { .. }) => Ok(None),
         Some(v) => ftd::e2(
@@ -332,14 +332,17 @@ pub fn decimal_optional(
     }
 }
 
-pub fn split(name: String, split_at: &str) -> crate::p1::Result<(String, String)> {
+pub fn split(name: String, split_at: &str) -> ftd::p1::Result<(String, String)> {
     let mut part = name.splitn(2, split_at);
     let part_1 = part.next().unwrap().trim();
     let part_2 = part.next().unwrap().trim();
     Ok((part_1.to_string(), part_2.to_string()))
 }
 
-pub fn reorder(p1: &[ftd::p1::Section], doc_id: &str) -> ftd::p1::Result<Vec<ftd::p1::Section>> {
+pub fn reorder(
+    p1: &[ftd::p1::Section],
+    doc_id: &str,
+) -> ftd::p1::Result<(Vec<ftd::p1::Section>, Vec<String>)> {
     fn is_kernel_component(comp: String) -> bool {
         if ["ftd.row", "ftd.column"].contains(&comp.as_str()) {
             return true;
@@ -352,11 +355,12 @@ pub fn reorder(p1: &[ftd::p1::Section], doc_id: &str) -> ftd::p1::Result<Vec<ftd
         new_p1: &mut Vec<ftd::p1::Section>,
         dependent_p1: Option<String>,
         inserted: &mut Vec<String>,
-        doc_id: &str,
-    ) {
+        doc: &ftd::p2::TDoc,
+        var_types: &[String],
+    ) -> ftd::p1::Result<()> {
         if let Some(p1) = dependent_p1 {
             if inserted.contains(&p1) {
-                return;
+                return Ok(());
             }
             if let Some(v) = p1_map.get(&p1) {
                 for sub_section in v.sub_sections.0.iter() {
@@ -368,18 +372,32 @@ pub fn reorder(p1: &[ftd::p1::Section], doc_id: &str) -> ftd::p1::Result<Vec<ftd
                         new_p1,
                         Some(sub_section.name.to_string()),
                         inserted,
-                        doc_id,
-                    );
+                        doc,
+                        var_types,
+                    )?;
                 }
-                if let Ok(root) = v.header.string(doc_id, v.line_number, "component") {
-                    if !is_kernel_component(root.to_string()) && !inserted.contains(&root) {
-                        reorder_component(p1_map, new_p1, Some(root), inserted, doc_id);
-                    }
+                let var_data = ftd::variable::VariableData::get_name_kind(
+                    &v.name,
+                    doc,
+                    v.line_number,
+                    var_types,
+                )?;
+                if !is_kernel_component(var_data.kind.to_string())
+                    && !inserted.contains(&var_data.kind)
+                {
+                    reorder_component(
+                        p1_map,
+                        new_p1,
+                        Some(var_data.kind),
+                        inserted,
+                        doc,
+                        var_types,
+                    )?;
                 }
                 new_p1.push(v.to_owned());
                 inserted.push(p1.to_string());
             }
-            return;
+            return Ok(());
         }
 
         for (k, v) in p1_map {
@@ -387,6 +405,22 @@ pub fn reorder(p1: &[ftd::p1::Section], doc_id: &str) -> ftd::p1::Result<Vec<ftd
                 continue;
             }
             for sub_section in v.sub_sections.0.iter() {
+                for (_, _, v) in sub_section.header.0.iter() {
+                    if v.contains(':') {
+                        let (name, _) = ftd::p2::utils::split(v.to_string(), ":")?;
+                        if inserted.contains(&name) || k == &name {
+                            continue;
+                        }
+                        reorder_component(
+                            p1_map,
+                            new_p1,
+                            Some(name.to_string()),
+                            inserted,
+                            doc,
+                            var_types,
+                        )?;
+                    }
+                }
                 if inserted.contains(&sub_section.name) || k == &sub_section.name {
                     continue;
                 }
@@ -395,42 +429,89 @@ pub fn reorder(p1: &[ftd::p1::Section], doc_id: &str) -> ftd::p1::Result<Vec<ftd
                     new_p1,
                     Some(sub_section.name.to_string()),
                     inserted,
-                    doc_id,
-                );
+                    doc,
+                    var_types,
+                )?;
             }
-            if let Ok(root) = v.header.string(doc_id, v.line_number, "component") {
-                if !is_kernel_component(root.to_string()) && !inserted.contains(&root) {
-                    reorder_component(p1_map, new_p1, Some(root), inserted, doc_id);
+            for (_, _, v) in v.header.0.iter() {
+                if v.contains(':') {
+                    let (name, _) = ftd::p2::utils::split(v.to_string(), ":")?;
+                    if inserted.contains(&name) || k == &name {
+                        continue;
+                    }
+                    reorder_component(
+                        p1_map,
+                        new_p1,
+                        Some(name.to_string()),
+                        inserted,
+                        doc,
+                        var_types,
+                    )?;
                 }
             }
+            let var_data =
+                ftd::variable::VariableData::get_name_kind(&v.name, doc, v.line_number, var_types)?;
+            if !is_kernel_component(var_data.kind.to_string()) && !inserted.contains(&var_data.kind)
+            {
+                reorder_component(
+                    p1_map,
+                    new_p1,
+                    Some(var_data.kind),
+                    inserted,
+                    doc,
+                    var_types,
+                )?;
+            }
 
             new_p1.push(v.to_owned());
             inserted.push(k.to_string());
         }
+        Ok(())
     }
 
+    let doc = ftd::p2::TDoc {
+        name: doc_id,
+        aliases: &Default::default(),
+        bag: &Default::default(),
+    };
+
     let mut p1_map: std::collections::BTreeMap<String, ftd::p1::Section> = Default::default();
     let mut inserted_p1 = vec![];
     let mut new_p1 = vec![];
     let mut list_or_var = vec![];
+    let mut var_types = vec![];
     for (idx, p1) in p1.iter().enumerate() {
+        let var_data =
+            ftd::variable::VariableData::get_name_kind(&p1.name, &doc, p1.line_number, &var_types);
         if p1.name == "import"
             || p1.name.starts_with("record ")
             || p1.name.starts_with("or-type ")
-            || p1.name.starts_with("list ")
             || p1.name.starts_with("map ")
-            || ftd::variable::VariableData::get_name_kind(&p1.name, doc_id, p1.line_number, true)
-                .is_ok()
         {
             inserted_p1.push(idx);
             new_p1.push(p1.to_owned());
-            if p1.name.starts_with("list ") {
-                let list = ftd::get_name("list", p1.name.as_str(), doc_id)?.to_string();
-                list_or_var.push(list);
-            }
-            if p1.name.starts_with("var ") {
-                let var = ftd::get_name("var", p1.name.as_str(), doc_id)?.to_string();
-                list_or_var.push(var);
+        }
+        if let Ok(ftd::variable::VariableData {
+            type_: ftd::variable::Type::Variable,
+            ref name,
+            ..
+        }) = var_data
+        {
+            inserted_p1.push(idx);
+            new_p1.push(p1.to_owned());
+            list_or_var.push(name.to_string());
+        }
+
+        if p1.name.starts_with("record ") {
+            let name = ftd::get_name("record", &p1.name, "")?;
+            var_types.push(name.to_string());
+        }
+
+        if p1.name.starts_with("or-type ") {
+            let name = ftd::get_name("or-type", &p1.name, "")?;
+            var_types.push(name.to_string());
+            for s in &p1.sub_sections.0 {
+                var_types.push(format!("{}.{}", name, s.name));
             }
         }
 
@@ -439,16 +520,25 @@ pub fn reorder(p1: &[ftd::p1::Section], doc_id: &str) -> ftd::p1::Result<Vec<ftd
             new_p1.push(p1.to_owned());
         }
 
-        if p1.name.starts_with("component ") {
-            p1_map.insert(
-                ftd::get_name("component", p1.name.as_str(), doc_id)?.to_string(),
-                p1.to_owned(),
-            );
+        if let Ok(ftd::variable::VariableData {
+            type_: ftd::variable::Type::Component,
+            ref name,
+            ..
+        }) = var_data
+        {
+            p1_map.insert(name.to_string(), p1.to_owned());
             inserted_p1.push(idx);
         }
     }
     let mut new_p1_component = vec![];
-    reorder_component(&p1_map, &mut new_p1_component, None, &mut vec![], doc_id);
+    reorder_component(
+        &p1_map,
+        &mut new_p1_component,
+        None,
+        &mut vec![],
+        &doc,
+        &var_types,
+    )?;
     new_p1.extend(new_p1_component);
 
     for (idx, p1) in p1.iter().enumerate() {
@@ -458,15 +548,51 @@ pub fn reorder(p1: &[ftd::p1::Section], doc_id: &str) -> ftd::p1::Result<Vec<ftd
         new_p1.push(p1.to_owned());
     }
 
-    Ok(new_p1)
+    Ok((new_p1, var_types))
 }
 
 pub fn properties(
-    properties_with_ref: &std::collections::BTreeMap<String, (crate::Value, Option<String>)>,
-) -> std::collections::BTreeMap<String, crate::Value> {
-    let mut properties: std::collections::BTreeMap<String, crate::Value> = Default::default();
+    properties_with_ref: &std::collections::BTreeMap<String, (ftd::Value, Option<String>)>,
+) -> std::collections::BTreeMap<String, ftd::Value> {
+    let mut properties: std::collections::BTreeMap<String, ftd::Value> = Default::default();
     for (k, (v, _)) in properties_with_ref {
         properties.insert(k.to_string(), v.to_owned());
     }
     properties
 }
+
+pub fn structure_header_to_properties(
+    s: &str,
+    arguments: &std::collections::BTreeMap<String, crate::p2::Kind>,
+    doc: &ftd::p2::TDoc,
+    line_number: usize,
+    p1: &ftd::p1::Header,
+) -> ftd::p1::Result<std::collections::BTreeMap<String, ftd::component::Property>> {
+    let (name, caption) = ftd::p2::utils::split(s.to_string(), ":")?;
+    match doc.get_thing(line_number, &name) {
+        Ok(ftd::p2::Thing::Component(c)) => ftd::component::read_properties(
+            line_number,
+            p1,
+            &if caption.is_empty() {
+                None
+            } else {
+                Some(caption)
+            },
+            &None,
+            "",
+            "",
+            &c.arguments,
+            arguments,
+            doc,
+            &Default::default(),
+            false,
+        ),
+        t => {
+            return ftd::e2(
+                format!("expected component, found: {:?}", t),
+                doc.name,
+                line_number,
+            )
+        }
+    }
+}
